#include <assert.h>
#include <math.h>
#include <stdio.h>
#include "types/dataset.h"
#include "adaboost.h"

AdaBoost::AdaBoost (size_t n_input, size_t n_output, size_t max)
    : Boosting(n_input, n_output, max)
{
}

AdaBoost::AdaBoost (AdaBoost &ag)
    : Boosting(ag)
{
}

AdaBoost* AdaBoost::copy()
{
    return new AdaBoost(*this);
}

REAL AdaBoost::train (REAL **x, REAL **y, size_t n, REAL *wgt)
{
    init_wgt = rv_alloc(n);
    if (wgt == NULL) {
        for (size_t i = 0; i < n; i++)
            init_wgt[i] = 1.0 / n;
    }
    else
        rv_copy(init_wgt, wgt, n);

    REAL err = Boosting::train(x, y, n, init_wgt);

    rv_free(init_wgt);
    return err;
}

void AdaBoost::update_weights (size_t k, REAL **x, REAL **y, size_t n)
{
    LearnModel *l = lm[k];
    BOOL *wrong_class = new BOOL[n];
    size_t i;

    assert(output_dim == l->get_n_output());
    REAL *out = rv_alloc(output_dim);
    REAL err = 0;
    for (i = 0; i < n; i++) {
        l->get_output(x[i], out);

        REAL max = out[0];
        size_t maxj = 0;
        for (size_t j = 1; j < output_dim; j++) {
            if (max < out[j]) {
                max = out[j];
                maxj = j;
            }
        }
        wrong_class[i] = (y[i][maxj] < 0);
        if (wrong_class[i])
            err += boost_wgt[i];
    }
    rv_free(out);

    printf("Weighted classification error: %g%%\n", err*100);

    REAL beta;
    if (err <= 0)
        beta = 1000;
    else
        beta = 1 / err - 1;

    if (err > 0.5) {
        /* do a reset */
        rv_copy(boost_wgt, init_wgt, n);
        lm_wgt[k] = 0;
    }
    else {
        lm_wgt[k] = log(beta);

        REAL bw_sum = 0;
        for (i = 0; i < n; i++) {
            if (wrong_class[i])
                boost_wgt[i] *= beta;
            bw_sum += boost_wgt[i];
        }

        assert(bw_sum != 0);
        for (i = 0; i < n; i++)
            boost_wgt[i] /= bw_sum;
    }
    delete wrong_class;
}
