#include <assert.h>
#include <math.h>
#include <stdio.h>
#include "types/dataset.h"
#include "adaboost_m2.h"
#include "classnn.h"

AdaBoost_M2::AdaBoost_M2 (size_t n_input, size_t n_output, size_t max)
    : Boosting(n_input, n_output, max)
{
}

AdaBoost_M2::AdaBoost_M2 (AdaBoost_M2 &ag)
    : Boosting(ag)
{
}

AdaBoost_M2* AdaBoost_M2::copy()
{
    return new AdaBoost_M2(*this);
}

REAL AdaBoost_M2::train (REAL **x, REAL **y, size_t n, REAL *wgt)
{
    boost_wgt = rv_alloc(n);
    if (wgt == NULL) {
        REAL p = 1.0 / n;
        for (size_t i = 0; i < n; i++)
            boost_wgt[i] = p;
    }
    else
        rv_copy(boost_wgt, wgt, n);

    assert(output_dim > 2);
    comp_wgt = rm_alloc(n, output_dim);
    for (size_t i = 0; i < n; i++) {
        REAL p = boost_wgt[i] / (output_dim - 1);
        REAL *cw = comp_wgt[i];
        REAL *yi = y[i];
        for (size_t j = 0; j < output_dim; j++, yi++, cw++)
            if (*yi > 0.9) *cw = 0;
            else *cw = p;
    }

    REAL err = train_sub(x, y, n);

    rm_free(comp_wgt, n);
    rv_free(boost_wgt);
    return err;
}

REAL AdaBoost_M2::train_sub (REAL **x, REAL **y, size_t n)
{
    assert(n_in_agg == 0 && n_model == 0);
    assert(lm_base != NULL); /* we need lm_base to create new hypotheses */

    //BOOL wgted_data = lm_base->support_weighted_data();
    //REAL **boost_x = NULL, **boost_y = NULL;
    //if (!wgted_data) {
    //    boost_x = (REAL**) malloc(n * sizeof(REAL*));
    //    boost_y = (REAL**) malloc(n * sizeof(REAL*));
    //}

    REAL err_sum = 0, wsum = 0;
    for (size_t i = 0; i < max_model; i++) {
        lm[i] = lm_base->copy();
        n_model++;
        printf("=== %s #%u / %u ===\n", version(), n_model, max_model);
        assert(lm[i] != NULL);

        /* dangerous */
        ((ClassNN*)lm[i])->set_weights(comp_wgt, n, y);
        REAL err;
        //if (!wgted_data) {
        //    printf("generating data ... "); fflush(stdout);
        //    resample(x, y, n, boost_wgt, boost_x, boost_y, n);
        //    printf("done.\n");
        //    err = lm[i]->train(boost_x, boost_y, n, NULL);
        err = lm[i]->train(x, y, n, NULL);
        //}
        //else err = lm[i]->train(x, y, n, boost_wgt);

        /* update boost_wgt, set up hypothesis wgt (lm_wgt) */
        update_weights(i, x, y, n);
        n_in_agg++;

        err_sum += err * lm_wgt[i];
        wsum += lm_wgt[i];
    }

    //if (!wgted_data) {
    //    free(boost_x);
    //    free(boost_y);
    //}
    //rv_free(boost_wgt);
    assert(wsum != 0);
    return (err_sum / wsum);
}

void AdaBoost_M2::update_weights (size_t k, REAL **x, REAL **y, size_t n)
{
    LearnModel *l = lm[k];
    size_t i, j;

    assert(output_dim == l->get_n_output());
    REAL *out = rv_alloc(output_dim);
    REAL **outd = rm_alloc(n, output_dim);
    REAL err = 0;
    for (i = 0; i < n; i++) {
        l->get_output(x[i], out);

        REAL *yi = y[i], oy = 0;
        for (j = 0; j < output_dim; j++) {
            if (*yi++ > 0.9) {
                oy = out[j];
                break;
            }
        }
        REAL *cw = comp_wgt[i], *od = outd[i];
        REAL *ou = out;
        for (j = output_dim; j; j--)
            err += (*cw++) * (*od++ = (*ou++) - oy);
    }
    rv_free(out);
    err = (2 + err) / 4;

    printf("Pseudo-loss: %g\n", err);
    assert(err < 0.5 && err >= 0);

    REAL log_beta;
    log_beta = log(1 / err - 1);

    lm_wgt[k] = log_beta;

    log_beta /= 4;
    REAL cw_sum = 0;
    for (i = 0; i < n; i++) {
        REAL *cw = comp_wgt[i], *od = outd[i];
        for (j = output_dim; j; j--) {
            *cw *= exp(log_beta * (*od++));
            cw_sum += *cw++;
        }
    }

    rm_free(outd, n);

    assert(cw_sum > 0);
    for (i = 0; i < n; i++) {
        REAL *cw = comp_wgt[i];
        for (j = output_dim; j; j--)
            *cw++ /= cw_sum;
    }
}
