#include <assert.h>
#include <stdio.h>
#include "average.h"

Average::Average (size_t n_input, size_t n_output, size_t max)
    : Aggregation(n_input, n_output, max)
{
}

Average::Average (Average &ag)
    : Aggregation(ag)
{
}

Average* Average::copy()
{
    return new Average(*this);
}

void Average::initialize ()
{
    while (n_model < max_model) {
        assert(lm_base != NULL);
        lm[n_model] = lm_base->copy();
        n_model++;
    }
    n_in_agg = n_model;

    assert(n_model > 0);
    for (size_t i = 0; i < n_model; i++)
        lm[i]->initialize();
}

void Average::aggregate (REAL *y)
{
    assert(n_in_agg > 0 && n_in_agg <= n_model);

    for (size_t i = 0; i < output_dim; i++) {
        y[i] = 0;
        for (size_t j = 0; j < n_in_agg; j++)
           y[i] += model_out[j][i];
        y[i] /= n_in_agg;
    }
}

REAL Average::train (REAL **x, REAL **y, size_t n, REAL *wgt)
{
    assert(wgt == NULL || support_weighted_data());

    assert(n_in_agg > 0 && n_in_agg <= n_model);
    REAL err = 0;
    for (size_t i = 0; i < n_in_agg; i++) {
        printf("=== %s #%u / %u ===\n", version(), i+1, n_in_agg);
        assert(lm[i] != NULL);
        err += lm[i]->train(x, y, n, wgt);
    }
    return (err / n_in_agg);
}
