#ifndef __CLASSNN_H__
#define __CLASSNN_H__
#define __LEARNING_MODEL_CLASSNN_H__

#include <stdio.h>
#include "types/feedforwardnn.h"

class ClassNN : public FeedForwardNN
{
 private:
    static const char name_version[] = "ClassNN 11/11/2001";

    REAL **tr_in, **tr_out, **val_in, **val_out;
    size_t n_tr, n_val;

    void alloc_space (size_t n);
    void free_space ();

 protected:
    REAL **comp_wgt, *sample_wgt;
    size_t wgt_n;

 protected:
    /* anyway, I want to specify which sample I am learning */
    REAL _error (size_t i, REAL *y);
    void _error_deriv (size_t i, REAL *y, REAL *deriv);
    virtual void log_error (size_t epoch, REAL err);

    virtual REAL epoch (REAL **x, REAL **y, size_t n, REAL *wgt);
    virtual REAL linesearch (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL gd_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL adapt_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL line_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL cg (REAL **x, REAL **y, size_t n, REAL *wgt);

 public:
    ClassNN (size_t n_input, REAL **tr_i, REAL **tr_o, size_t trn,
             REAL **val_in, REAL **val_out, size_t valn);
    ClassNN (ClassNN &nn);
    virtual ~ClassNN();
    virtual const char* version () { return name_version; }

    virtual ClassNN* copy ();
    void set_weights (REAL **c_wgt, size_t n, REAL **y);
    virtual REAL train (REAL **x, REAL **y, size_t n, REAL *wgt = NULL);
};

#else /* def __CLASSNN_H__ */

#ifndef __LEARNING_MODEL_CLASSNN_H__
#error This header file conflicts with another "classnn.h" file.
#endif

#endif
