#include <assert.h>
#include "nnlayer_momentum.h"

NNLayer_Momentum::NNLayer_Momentum (size_t input, size_t neuron)
    : NNLayer(input, neuron)
{
    set_momentum_ratio(0.1);
}

NNLayer_Momentum::NNLayer_Momentum (size_t input, size_t neuron, REAL r)
    : NNLayer(input, neuron)
{
    set_momentum_ratio(r);
}

NNLayer_Momentum::NNLayer_Momentum (NNLayer_Momentum &ml)
    : NNLayer(ml)
{
    momentum_ratio = ml.momentum_ratio;
}

NNLayer_Momentum* NNLayer_Momentum::copy ()
{
    return new NNLayer_Momentum(*this);
}

void NNLayer_Momentum::set_momentum_ratio (REAL r)
{
    assert(r >= 0);
    momentum_ratio = r;
}

void NNLayer_Momentum::update_weight (REAL learn_rate)
{
    size_t i, j;

    for (i = 0; i < n_neuron; i++) {
        for (j = 0; j <= n_input; j++) {
            weight[i][j] -= learn_rate * wdelta[i][j];
            wdelta[i][j] *= momentum_ratio;
        }
    }
}
