#include <assert.h>
#include <stdio.h>
#include "bagging.h"

Bagging::Bagging (size_t n_input, size_t n_output, size_t max)
    : Aggregation(n_input, n_output, max)
{
}

Bagging::Bagging (Bagging &ag)
    : Aggregation(ag)
{
}

Bagging* Bagging::copy()
{
    return new Bagging(*this);
}

void Bagging::initialize ()
{
    size_t i;
    for (i = 0; i < n_model; i++) {
        assert(lm[i] != NULL);
        delete(lm[i]);
    }

    assert(lm_base != NULL);
    lm_base->initialize();
    for (i = 0; i < max_model; i++)
        lm[i] = lm_base->copy();

    n_in_agg = n_model = max_model;
}

void Bagging::aggregate (REAL *y)
{
    assert(n_in_agg > 0 && n_in_agg <= n_model);

    for (size_t i = 0; i < output_dim; i++) {
        y[i] = 0;
        for (size_t j = 0; j < n_in_agg; j++)
           y[i] += model_out[j][i];
        y[i] /= n_in_agg;
    }
}

REAL Bagging::train (REAL **x, REAL **y, size_t n, REAL *wgt)
{
    assert(n_in_agg > 0 && n_in_agg <= n_model);
    assert(wgt == NULL || support_weighted_data());

    REAL **boost_x, **boost_y;
    assert(n > 0);
    boost_x = (REAL**) malloc(n * sizeof(REAL*));
    boost_y = (REAL**) malloc(n * sizeof(REAL*));

    REAL err = 0;
    for (size_t i = 0; i < n_in_agg; i++) {
        printf("=== %s #%u / %u ===\n", version(), i+1, n_in_agg);
        printf("generating data ... "); fflush(stdout);
        resample(x, y, n, wgt, boost_x, boost_y, n);
        printf("done.\n");
        assert(lm[i] != NULL);
        err += lm[i]->train(boost_x, boost_y, n);
    }

    free(boost_x);
    free(boost_y);
    return (err / n_in_agg);
}
