#ifndef __DATASET_H__
#define __DATASET_H__
#define __COMMON_TYPES_DATASET_H__

#include "types.h"
#include "vecmat.h"

DECLARE_ALLOC_FREE_FUNC(REAL, r);

/* when data appended into a matrix, we need to resize the matrix... */
#define MATRIX_SIZE_INC_STEP    10

class Matrix
{
private:
    REAL **x;
    size_t size, dim, alloc_size;

public:
    Matrix ();
    virtual ~Matrix ();

    size_t getSize ()   { return size; }    /* How many samples */
    size_t getDimension ()  { return dim; }     /* Dimension of vector */
    REAL** getBuffer () { return x; }

    BOOL setData (size_t len, size_t dimen, REAL **buf);
    BOOL appendData (size_t len, size_t dimen, REAL **buf);
//  BOOL appendMatrix (Matrix &cm);

//  DataSet &pickWithReplace();
//  void addNoise();        /* add noise */

protected:
    void init ();
    void freeBuffer ();
    BOOL expandSize (size_t len, size_t dimen);
};

class DataSet
{
private:
    Matrix *x, *y;

public:
    DataSet();
    virtual ~DataSet();

    size_t getDataSize ()   { return x->getSize() ; }   /* How many samples */
    size_t getInputDim ()   { return x->getDimension(); }   /* dim of input */
    size_t getOutputDim()   { return y->getDimension(); }
    REAL** getInputBuf ()   { return x->getBuffer(); }
    REAL** getOutputBuf()   { return y->getBuffer(); }

    BOOL setData (size_t len, size_t in, size_t out, REAL** x, REAL** y);
    BOOL appendData (size_t len, size_t in, size_t out, REAL** x, REAL** y);
    BOOL combineData (Matrix &in, Matrix &out);

    size_t readDataFile (size_t len, size_t in, size_t out, FILE *f);

//  DataSet &pickWithReplace();
//  void addNoise();        /* add noise */
protected:
    void init ();
    void freeAll ();
};

#else /* def __DATASET_H__ */

#ifndef __COMMON_TYPES_DATASET_H__
#error This header file conflicts with another "dataset.h" file.
#endif

#endif
