#ifndef __FEEDFORWARDNN_H__
#define __FEEDFORWARDNN_H__
#define __LEARNING_MODEL_FEEDFORWARDNN_H__

#include <stdio.h>
#include "types.h"
#include "learnmodel.h"
#include "nnlayer.h"

#define MAX_N_LAYER     10
class FeedForwardNN : public LearnModel
{
 private:
    static const char name_version[] = "FeedForwardNN 11/02/2001";

    void alloc_space (size_t n_input);
    void free_space ();

 public:
    enum TRAIN_METHOD {
        GRADIENT_DESCENT,
        LINE_SEARCH,
        CONJUGATE_GRADIENT,
        ADAPTIVE_LEARNING_RATE
    };

 protected:
    size_t n_layer;
    size_t layer_size[MAX_N_LAYER+1];
    NNLayer *layer[MAX_N_LAYER+1];
    REAL *output[MAX_N_LAYER+1], *deriv[MAX_N_LAYER+1];

    BOOL online_learn;
    REAL learn_rate, min_err;
    size_t max_run;
    TRAIN_METHOD train_method;

 protected:
    virtual REAL error (REAL *out, REAL *y);
    inline REAL _error (REAL *y);
    virtual void error_deriv (REAL *y, REAL *deriv);

    virtual void log_error (size_t epoch, REAL err);

    virtual REAL epoch (REAL **x, REAL **y, size_t n, REAL *wgt);
    virtual REAL linesearch (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL gd_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL adapt_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL line_train (REAL **x, REAL **y, size_t n, REAL *wgt);
    REAL cg (REAL **x, REAL **y, size_t n, REAL *wgt);

 public:
    FeedForwardNN (size_t n_input);
    FeedForwardNN (FeedForwardNN &nn);
    virtual ~FeedForwardNN();
    virtual const char* version () { return name_version; }

    virtual BOOL save_model (FILE *f);
    virtual BOOL load_model (FILE *f);

    virtual FeedForwardNN* copy ();
    virtual BOOL support_weighted_data () { return TRUE; }
    virtual void initialize ();
    virtual REAL train (REAL **x, REAL **y, size_t n, REAL *wgt = NULL);
    virtual void get_output (REAL *x, REAL *y);

 public:
    void add_layer (NNLayer *nl);
    void set_batch_mode (BOOL b = TRUE) { online_learn = !b; }
    void set_train_method (TRAIN_METHOD m) { train_method = m; }
    void set_parameter (REAL learn_rate, REAL min_err, size_t max_run);
};

#else /* def __FEEDFORWARDNN_H__ */

#ifndef __LEARNING_MODEL_FEEDFORWARDNN_H__
#error This header file conflicts with another "feedforwardnn.h" file.
#endif

#endif
