#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include "dataset.h"
#include "learnmodel.h"

LearnModel::LearnModel (size_t n_input, size_t n_output)
{
    input_dim = n_input;
    output_dim = n_output;
    logf = NULL;
}

LearnModel::LearnModel (LearnModel &lm) : Object(lm)
{
    input_dim  = lm.input_dim;
    output_dim = lm.output_dim;
    logf       = lm.logf;       /* is this reasonable? */
}

LearnModel::~LearnModel ()
{
    /* empty */
}

REAL LearnModel::error (REAL *out, REAL *y)
{
    REAL err = 0;
    for (size_t i = 0; i < output_dim; i++) {
        REAL dif = (*out++) - (*y++);
        err += dif * dif;
    }
    return err;
}

REAL LearnModel::train2 (DataSet &data)
{
    assert(data.getInputDim() == input_dim);
    assert(data.getOutputDim() == output_dim);
    return
        train(data.getInputBuf(), data.getOutputBuf(), data.getDataSize());
}

REAL LearnModel::test (DataSet &data)
{
/*
    REAL **sam_in = data.getInputBuf();
    REAL **sam_out = data.getOutputBuf();
    assert(sam_in != NULL && sam_out != NULL);
*/

    REAL total_err = 0;
    size_t i, n_sample;
    i = n_sample = data.getDataSize();
/*  while (i--) {
        get_output(*sam_in, out);
        total_err += error(out, *sam_out);
        sam_in++; sam_out++;
    }*/
    return total_err / n_sample;
}

void LearnModel::get_output (DataSet &data)
{
    REAL **in = data.getInputBuf();
    REAL **out = data.getOutputBuf();
    assert(in != NULL && out != NULL);
    assert(data.getInputDim() == input_dim
        && data.getOutputDim() == output_dim);

    size_t i = data.getDataSize();
    while (i--) {
        get_output(*in, *out);
        in++; out++;
    }
}

/* In support of aggregation.
 * the wgt things by default are disabled
 */
BOOL LearnModel::support_weighted_data ()
{
    return FALSE;
}
