#ifndef __NNLAYER_H__
#define __NNLAYER_H__
#define __LEARNING_MODEL_NNLAYER_H__

#include <stdio.h>
#include "types.h"
#include "object.h"

typedef REAL WEIGHT;

class NNLayer : public Object
{
 private:
    static const char name_version[] = "NNLayer 10/26/2001";
    REAL stored_sigmoid;

    void alloc_space ();
    void free_space ();

 protected:
    size_t n_neuron, n_input;
    WEIGHT winit_min, winit_max;
    WEIGHT **weight, **wbackup;
    WEIGHT **wdelta;  /* update by: weight -= lr * delta */
    WEIGHT **wdir;  /* update by: weight -= lr * wdir */
    WEIGHT **wdel_old;
    REAL *sig_der;  /* f'(wsum) */

 public:
    NNLayer (size_t num_input, size_t num_neuron);
    NNLayer (NNLayer &l);
    virtual ~NNLayer ();
    virtual const char* version () { return name_version; }

    virtual BOOL save_model (FILE *f);
    virtual BOOL load_model (FILE *f);
    virtual NNLayer* copy ();

    size_t get_n_neuron () { return n_neuron; }
    size_t get_n_input () { return n_input; }

    void feed_forward (REAL *input, REAL *output);
    void back_propagate (REAL *input, REAL *derin, REAL *derout);

 public:
    /* for line search */
    void backup_weight ();
    void restore_weight ();
    void temp_update_weight (REAL step);

    /* norm square of gradient */
    REAL g_norm2 ();
    REAL g_dot_old ();
    void cg_update (REAL beta);

 public:
    virtual void initialize ();
    void initialize (REAL min, REAL max);
    virtual void update_weight (REAL learn_rate);
    virtual REAL sigmoid (REAL x);
    virtual REAL sigmoid_deriv (REAL x);
};

#else /* def __NNLAYER_H__ */

#ifndef __LEARNING_MODEL_NNLAYER_H__
#error This header file conflicts with another "nnlayer.h" file.
#endif

#endif
