/* vecmat.h -- Define types/functions for vectors and matrix
 *
 * Copyright (C) 2001 Ling Li
 * ling@caltech.edu
 */

#ifndef __VECMAT_H__

#define __VECMAT_H__
#define __COMMON_TYPES_VECMAT_H__

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "types.h"

/* NOTE: when including DECLARE_ALLOC_FREE_FUNC,
         please also use extern "C" {} */
/* allocate/free memory for vectors, matrix */
/* Probability Matrix. m vectors, each with length n */
#define DECLARE_ALLOC_FREE_FUNC(base, ty)               \
    base* ty##v_alloc (size_t n);                       \
    void ty##v_free (base* ty##v);                      \
    void ty##v_clear (base* ty##v, size_t n);           \
    void ty##v_copy (base* dst, base* src, size_t n);   \
    base** ty##m_alloc (size_t m, size_t n);            \
    void ty##m_free (base** ty##m, size_t m)
#define DEFINE_ALLOC_FREE_FUNC(base, ty)                \
    base* ty##v_alloc (size_t n) {                      \
        base* ty##v = (base*) malloc(sizeof(base) * n); \
        assert(n > 0);                                  \
        assert(ty##v != NULL);                          \
        return ty##v;                                   \
    }                                                   \
    void ty##v_free (base* ty##v) {                     \
        assert(ty##v != NULL);                          \
        free(ty##v);                                    \
    }                                                   \
    void ty##v_clear (base* ty##v, size_t n) {          \
        assert(ty##v != NULL);                          \
        assert(n > 0);                                  \
        while (n--)                                     \
            *ty##v++ = 0;                               \
    }                                                   \
    void ty##v_copy (base* ty##v_dst, base* ty##v_src,  \
                     size_t n) {                        \
        assert(ty##v_dst != NULL);                      \
        assert(ty##v_src != NULL);                      \
        assert(n > 0);                                  \
        while (n--)                                     \
            *ty##v_dst++ = *ty##v_src++;                \
    }                                                   \
    base** ty##m_alloc (size_t m, size_t n) {           \
        size_t i;                                       \
        base** ty##m = (base**) malloc(sizeof(base*)*m);\
        assert(n > 0 && m > 0);                         \
        assert(ty##m != NULL);                          \
        for (i = 0; i < m; i++)                         \
            ty##m[i] = ty##v_alloc(n);                  \
        return ty##m;                                   \
    }                                                   \
    void ty##m_free (base** ty##m, size_t m) {          \
        assert(ty##m != NULL);                          \
        assert(m > 0);                                  \
        while (m--)                                     \
            ty##v_free(ty##m[m]);                       \
        free(ty##m);                                    \
    }

#else /* __VECMAT_H__ */

#ifndef __COMMON_TYPES_VECMAT_H__
#error This header file conflicts with another "vecmat.h" file.
#endif

#endif
